/*****************************************************************************\
 *
 * 3ds2iv.cpp
 *
 * Updated: 14.12.2005
 * 
 * 3ds2iv    - call function for creates scene graph 
 *             and sends a root to the ExaminerViewer.
 *
 * Author: Jaroslav Pribyl (xpriby05 _AT stud.fit.vutbr.cz)
 *
 * Contributors:
 *
 * ----------------------------------------------------------------------------
 *
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 * This source code is offered for use in the public domain.
 * You may use, modify or distribute it freely.
 *
 * This source code is distributed in the hope that it will be useful but
 * WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 * DISCLAIMED.  This includes but is not limited to warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * If you find the source code useful, authors will kindly welcome
 * if you give them credit and keep their names with their source code,
 * but do not feel to be forced to do so.
 *
\*****************************************************************************/

#include <iostream>

// use windows or linux viewer libraries
#if defined(__WIN32__) || defined(_WIN32)
  #include <Inventor/Win/SoWin.h>
  #include <Inventor/Win/viewers/SoWinExaminerViewer.h>
#else
  #include <Inventor/Qt/SoQt.h>
  #include <Inventor/Qt/viewers/SoQtExaminerViewer.h>
#endif

#include "common.h"
#include "sceneClass.h"
#include "geometryClass.h"

// write easy help for this program
void printHelp()
{
  std::cout << "Usage: 3ds2iv [options] infile [outfile]\n";
  std::cout << "   -e      : Use Examine viewer to show the model and create outfile\n";
  std::cout << "   -f      : Compute normals using flat shading\n";
  std::cout << "   -h      : Print this message (help)\n";
  std::cout << "   -s      : Force two sided materials\n";
  std::cout << "   -t      : Do not apply textures\n";
  std::cout << "   -v      : Do not create outfile, only use Examine viewer to show the model\n";
  std::cout << "If no input or output file name is specified, stdin and stdout will be used.\n";
}

void parseCommandLine(int argc, char **argv, params &param) {
 // command line parser
  if( argc <= 1 )
  {
    // bad command line parameters causes help printing
    printHelp();
    exit(0);
  }
  else
  {
    int q = 1; // for go thru command line parameters (count)
    // parse all parameters and check correct syntax
    while( q < argc && argv[q][0] == '-' ) 
    {
        int p = 1;
        // check if parameter isn't empty
        while(argv[q][p] != '\0')
        {
          switch( argv[q][p]) {            
            // convert and use ExViewer to show the model
            case 'e' :
              param.viewer = true;
            break;
            // force faceted shading model
            case 'f' :
              param.faceted = true;
            break;
            // print easy help for this program
            case 'h' :
              printHelp();
              exit(0);
            break;
            // force two sided material for all object included in input 3ds file
            case 's' :
              param.twoSided = true;
            break;
            // force no texture is applicated
            case 't' :
              param.noText = true;
            break;
            // only use ExViewer to show the model, for faster moedl view (no conversion is
            // making)
            case 'v' :
              param.noConv = true;
            break;
            // in default print easy help for this program
            default:
              printHelp();
              exit(0);
            break;
          }
          p++;
        }
        q++;
    }

    // No more parameters in command line = no input file
    if( q < argc )
    {
      // Only input file entered, also output file'll be stdout
      if( (argc - q) == 1 )
      {
        // check correct syntax
        if( argv[q][0] != '-' )
        {
          param.infile = argv[q];
          param.outfile = "";
        }
        else
        {
          printHelp();
          exit(0);
        }
      }
      else if( (argc - q) == 2 ) // both input file and output file are entered
      {
        if( argv[q][0] != '-' && argv[q + 1][0] != '-')
        {
          param.infile = argv[q];
          param.outfile = argv[q + 1];
        }
        else
        {
          printHelp();
          exit(0);
        }
      }
      else
      { 
        // bad input parameters causes help printing
        printHelp();
        exit(0);
      }
    }
    else
    {
      // bad input parameters cause help printing
      printHelp();
      exit(0);
    }
  }
}


int main (int argc, char **argv) {

  // Initialize Inventor a Qt utility for windows or for linux
  #if defined(__WIN32__) || defined(_WIN32)
    HWND myWindow = SoWin::init("Semestralni projekt: 3ds -> Open Inventor");
    SoWinExaminerViewer *myViewer = new SoWinExaminerViewer(myWindow);
  #else
    QWidget *myWindow = SoQt::init("Semestralni projekt: 3ds -> Open Inventor");
    SoQtExaminerViewer *myViewer = new SoQtExaminerViewer(myWindow);
  #endif
  
  // create structure to store command line parameters
  params param;
  // the application's window
  param.mainWin = argv[0];

  parseCommandLine(argc, argv, param);
 
  sceneClass *scene = new sceneClass(param);

  // create scene graph and return root back in to the viewer
  myViewer->setSceneGraph(scene->createScene());

  // use viewer for -e and -v parameters
  if( param.viewer || param.noConv ) {
    myViewer->show();

    // show the results for windows and linux too
    #if defined(__WIN32__) || defined(_WIN32)
      SoWin::show(myWindow);
      SoWin::mainLoop();
    #else
      SoQt::show(myWindow);
      SoQt::mainLoop();
    #endif
  }

  scene->deleteSceneGraph();
  delete scene;


  return(0);
}
