/*****************************************************************************\
 *
 * sceneClass.cpp
 *
 * Updated: 14.12.2005
 * 
 * sceneClass - create scene and can return the root node whitch represent it
 *
 * Author: Jaroslav Pribyl (xpriby05 _AT stud.fit.vutbr.cz)
 *
 * Contributors:
 *
 * ----------------------------------------------------------------------------
 *
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 * This source code is offered for use in the public domain.
 * You may use, modify or distribute it freely.
 *
 * This source code is distributed in the hope that it will be useful but
 * WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 * DISCLAIMED.  This includes but is not limited to warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * If you find the source code useful, authors will kindly welcome
 * if you give them credit and keep their names with their source code,
 * but do not feel to be forced to do so.
 *
\*****************************************************************************/

#include <iostream>

#include "sceneClass.h"
#include "geometryClass.h"
#include "materialClass.h"
#include "textureClass.h"
#include "keyFrameClass.h"

/*
 * This function read all object from database and
 * create scene as a scene graph
 */
SoSeparator *sceneClass::createScene() {
  
  //for all meshes in database
  for(int i = 0; i != meshList->count; i++)
  {
    // Load object based on his name
    model.getMesh(meshList->list[i].name, &mesh);
         
    // get instance count and perform his copyiing
    int count = model.getMeshInstanceCount(meshList->list[i].name);

    // for models with no animation
    int noKeyframe = (count == 0 );

    for( int j = 0; j < count + noKeyframe; j++ ) {    
      
      // create separator for new mesh
      meshSet = new SoSeparator();

      // when mesh contain negative scale, this variable will be set
 	    int clockshape = 0;

      if( !noKeyframe ) {
        model.getKeyFrameInstanceByIndex(meshList->list[i].name, j, &kfmesh);

        keyFrameClass *keyFrameCls = new keyFrameClass(this, j);
        keyFrameCls->performKFTransformations(&clockshape);
        delete keyFrameCls;
      }

      // create mesh's geometry
      geometryClass *geometryCls = new geometryClass(this);

      // if there is defined at least one material for this mesh
      if( mesh->nmats != 0 )
      {
        // create the scene step by step based on number of mnats in this mesh
        for(int mat_id = 0; mat_id != mesh->nmats; mat_id++)
        {
          // separately parts of object
          materialSet = new SoSeparator();
          // actualization for materialSet 
          geometryCls->initBit(materialSet);
          // load name for this object's part
          model.getMaterial(mesh->matarray[mat_id].name, &material);
          // create class for material realizations
          materialClass *materialCls = new materialClass(this);
          // add material to scene
          materialCls->addMaterial(clockshape);

          if( !param.noText )
          {
            // create textures
            textureClass *textureCls = new textureClass(this);
            textureCls->addTexture();
            delete textureCls;
          }       

          // create object's geometry objektu
          geometryCls->createGeometry(mat_id, material->shading);

          // put object's parts together
          meshSet->addChild(materialSet);
          // release resources enagnged with material
          delete materialCls;        
        }

        // put scene parts (objects) together
        root->addChild(meshSet);
      }
      else {
        // separately parts of object
        materialSet = new SoSeparator();
        // materialSet actualization
        geometryCls->initBit(materialSet);
        // create class for realizatize material
        materialClass *materialCls = new materialClass(this);
        // realize material (defult)
        materialCls->addMaterial(clockshape);
        // create object's geometry, material index is NULL (no material was defined)
        geometryCls->createGeometry(0, 0);
        // put object's parts together
        meshSet->addChild(materialSet);
        // put scene's parts (objects) together
        root->addChild(meshSet);
        delete materialCls;
      }
      delete geometryCls;
    }
  }
   
  // if there is set only view parameter, write action isn't performed
  if( !param.noConv )
  {
    // Create iv file from scene graph  
    SoWriteAction myAction;
  
    // set output file name from parameters
    // if no aoutput fil is typed, std::out will be used
    if( param.outfile != "" )
      myAction.getOutput()->openFile(param.outfile.c_str());
    
    // set text output instead of binary
    myAction.getOutput()->setBinary(FALSE);
    // apply this action on to root node
    myAction.apply(root);
    // close output text file
    myAction.getOutput()->closeFile();
  }

  //return the root node because the ExViewer need it
  return root;
}

/*
 * Function return a mesh structure stored in sceneClass
 */
meshLib3 *sceneClass::getMesh() {
  return mesh;
};

/*
 * Function return a material structure stored in sceneClass
 */
materialLib3 *sceneClass::getMaterial() {
  return material;
};

/*
 * Function return a command line parameters stored in sceneClass
 */
params sceneClass::getParam() {
  return param;
};

/*
 * Function return actual proccessed mesh separator
 */
SoSeparator *sceneClass::getMeshSet() {
  return(meshSet);
};

/*
 * Function return actual proccessed material separator
 */
SoSeparator *sceneClass::getMaterialSet() {
  return(materialSet);
};

/*
 * Function return kfmesh structure stored in  sceneClass
 */
kfmeshLib3 *sceneClass::getKfmesh() {
  return(kfmesh);
};

lib3ds *sceneClass::getModel() {
  return(&model); 
};

/*
 * Constructor
 */
sceneClass::sceneClass(params Param)
{
  meshList  = NULL;
  mesh      = NULL;
  material  = NULL;
  kfmesh	  = NULL;

  param = Param;

  // create database from 3ds file using lib3ds library

  //TODO: make better library error exception handler
  if( model.createModel((char *) param.infile.c_str()) ) {
     // Get names of all meshes in database
    model.getMeshList(&meshList);
  }
  else
  {
    std::cerr << "Cannon create database from this 3ds file.\n";
    exit(0);
  }

  // find out min and max line width for wire shading models
  glutCreateWindow(param.mainWin.c_str());
  GLfloat info[2];
  glGetFloatv(GL_LINE_WIDTH_RANGE, info);

  // store found min and max values for wire shading
  minLW = info[0];
  maxLW = info[1];

  // create a root node and refer it
  root = new SoSeparator();
  root->ref();
}

void sceneClass::deleteSceneGraph() {
  // dereferencing root scene's node
  if( root != NULL ) root->unref();
  // free model's database from memory
  model.releaseModel();
}


/*
 * Destructor is empty.
 */
sceneClass::~sceneClass() { 
}
