/*****************************************************************************\
 *
 * smoothGroupsClass.cpp
 *
 * Updated: 14.12.2005
 * 
 * geometryClass  - working with smoothing groups
 *
 * Author: Jaroslav Pribyl (xpriby05 _AT stud.fit.vutbr.cz)
 *
 * Contributors:
 *
 * ----------------------------------------------------------------------------
 *
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 * This source code is offered for use in the public domain.
 * You may use, modify or distribute it freely.
 *
 * This source code is distributed in the hope that it will be useful but
 * WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 * DISCLAIMED.  This includes but is not limited to warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * If you find the source code useful, authors will kindly welcome
 * if you give them credit and keep their names with their source code,
 * but do not feel to be forced to do so.
 *
\*****************************************************************************/

#include "smoothGroupsClass.h"


/*
 * Constructor
 */
smoothGroupsClass::smoothGroupsClass(meshLib3 *Mesh) {
  mesh = Mesh;
  smoothGroupList = NULL;
  augumentVertices = new SbList<tSmoothGroups> [mesh->nvertices];
}

/*
 * Destructor
 */
smoothGroupsClass::~smoothGroupsClass() {
  if( smoothGroupList != NULL ) {
    delete smoothGroupList;
    smoothGroupList = NULL;
  }

  if( augumentVertices != NULL ) {
    delete [] augumentVertices;
    augumentVertices = NULL;
  }
}

/*
 * This function return array with augumented vertices for use in normall class
 */
SbList<tSmoothGroups> *smoothGroupsClass::getAugumentVertices() {
  return(augumentVertices);
};

/*
 * This function return an integer number, which represents smoothing groups list
 */
int smoothGroupsClass::getIntSmoothingGroup(unsigned long int group)
{
  // if smoothing groups are defined in actual mesh, get face's smooth group
  if( mesh->smootharray != NULL )
    group = mesh->smootharray[group];
  else
    group = 0; // this face with group index is outside any smoothing groups

  return(group);
}

/*
 * This function return List. This List contain smgroups number, on this face belongch
 * Parameter group - 32 bits ... determining smooth groups for the face 
 */
SbIntList *smoothGroupsClass::getSmoothingGroups(unsigned long int group) {
  // create new integer list and delete the old one
  if( smoothGroupList != NULL ) {
    delete smoothGroupList;
    smoothGroupList = NULL;
  }

  smoothGroupList = new SbIntList;
  
  // check if there is defined one smoothing group array
  if( mesh->smootharray != NULL )
  {
    group = mesh->smootharray[group];
    // no smoothing group is used for face with index "group"
    if( group == 0 ) 
      smoothGroupList->append(0);
  }
  else
    smoothGroupList->append(0); // add to smooth groups list zero (it means no smg)

  // go thru all 32 bits and append found smooth groups to list
  int bit = 0;
  while(group != 0)
  {
    // if actual bit is one, append smooth group number to list
    if( (group & 0x1) == 1)
      smoothGroupList->append(bit + 1);

    // move to next smooth group
    group >>= 1;
    bit++;
  }
   
  // return list containing all smoothing groups which the face belong
  return(smoothGroupList);
}

/*
 * We find out and put together lists, with at least one smooth group
 */
void smoothGroupsClass::findSmoothGroupsUnion(int size, SbList<tSmoothGroups> *auxIndexes2)
{    
  int ai2size = auxIndexes2->operator[](size).smgIdentifier;
  // for equals smoothing groups, perform its conjunction
  int i;
  for (i = 0; i < size; i++)
    if( ai2size & auxIndexes2->operator[](i).smgIdentifier)
    {      
      auxIndexes2->operator[](i).smgIdentifier  = ai2size | auxIndexes2->operator[](i).smgIdentifier;
      auxIndexes2->operator[](size).smgIdentifier  = ai2size | auxIndexes2->operator[](i).smgIdentifier;
      continue;
    }
}


/* 
 * For all vertex add information about his smoothing group and face index which this vertex belong
 */
void smoothGroupsClass::performAugumentation() {

  SbVec3f       point;
  SbBSPTree     *strom;
    
  strom = new SbBSPTree;

  // duplicate array for later store pointer for augumentVertices and auxIndexes2 array swapping
  SbList<tSmoothGroups> *auxIndexes2 = new SbList<tSmoothGroups> [mesh->nvertices];
  //SbList<tSmoothGroups> *auxIndexes2 = new SbList<tSmoothGroups> [mesh->nvertices];
  int *auxIndexes3 = new int[mesh->nvertices];

  int k;  
  for(k = 0; k < mesh->nfaces; k++)
  {
    tSmoothGroups bit;
    // face index which is in processing
    bit.faceIndex = k;
    // sm groups, for this face
    bit.smgIdentifier = getIntSmoothingGroup(k);

    point.setValue( (float) mesh->vertexarray[mesh->facearray[k].v1].x,
                    (float) mesh->vertexarray[mesh->facearray[k].v1].y,
                    (float) mesh->vertexarray[mesh->facearray[k].v1].z);

    // insert item in to BSP tree and the returned index tell us, where this first vertex will be stored
    int idxvx = strom->addPoint( (const SbVec3f &) point);
    int size = augumentVertices[idxvx].getLength();

    augumentVertices[idxvx].append(bit);
    auxIndexes2[idxvx] = augumentVertices[idxvx];

    findSmoothGroupsUnion(size, &augumentVertices[idxvx]);

    auxIndexes3[mesh->facearray[k].v1] = idxvx;
    
    // eaqual upper code, but for second triangle's vertex
    point.setValue( (float) mesh->vertexarray[mesh->facearray[k].v2].x,
                    (float) mesh->vertexarray[mesh->facearray[k].v2].y,
                    (float) mesh->vertexarray[mesh->facearray[k].v2].z);

    idxvx = strom->addPoint( (const SbVec3f &) point);
    size = augumentVertices[idxvx].getLength();
    augumentVertices[idxvx].append(bit);
    auxIndexes2[idxvx] = augumentVertices[idxvx];

    findSmoothGroupsUnion(size, &augumentVertices[idxvx]);

    auxIndexes3[mesh->facearray[k].v2] = idxvx;

    // eaqual upper code, but for thirth triangle's vertex
    point.setValue( (float) mesh->vertexarray[mesh->facearray[k].v3].x,
                    (float) mesh->vertexarray[mesh->facearray[k].v3].y,
                    (float) mesh->vertexarray[mesh->facearray[k].v3].z);

    idxvx = strom->addPoint( (const SbVec3f &) point);
    size = augumentVertices[idxvx].getLength();
    augumentVertices[idxvx].append(bit);
    auxIndexes2[idxvx] = augumentVertices[idxvx];

    findSmoothGroupsUnion(size, &augumentVertices[idxvx]);
    
    auxIndexes3[mesh->facearray[k].v3] = idxvx;
  }

  // go thru all triangles one more time and map from auxindexes2 array into augumentVertices array
  for(k = 0; k < mesh->nfaces; k++)
  {
    augumentVertices[mesh->facearray[k].v1] = auxIndexes2[auxIndexes3[mesh->facearray[k].v1]];
    augumentVertices[mesh->facearray[k].v2] = auxIndexes2[auxIndexes3[mesh->facearray[k].v2]];
    augumentVertices[mesh->facearray[k].v3] = auxIndexes2[auxIndexes3[mesh->facearray[k].v3]];
  }

  delete strom;
  delete [] auxIndexes3;
  delete [] auxIndexes2;
}